///////////////////////////// A Color-Sensing Drawing Robot //////////////////////
//////////////////////////////////// Made By People //////////////////////////////

// Any code comment mentioning Sparkfun was adapted for this project from this website: https://learn.sparkfun.com/tutorials/apds-9960-rgb-and-gesture-sensor-hookup-guide#introduction//

#include <Wire.h>
#include <SparkFun_APDS9960.h>  // Arduino Library for the Color sensor that needs to be downloaded from SparkFun's website (https://learn.sparkfun.com/tutorials/apds-9960-rgb-and-gesture-sensor-hookup-guide#arduino-library-installation)
#include <Servo.h>              // Activates the built-in arduino library for servo motors
Servo myservo;                  // Declaring a servo motor

int AIN2 = 9;    // The pin that moves the "left motor" of our build clockwise. Your result may vary if you change up aspects of your physical build or code.
int AIN1 = 10;   // The pin that moves the "left motor" of our build counterclockwise.
int PWMA1 = 8;   // Essentially the power supply for the "left motor" regulating the amount of power a motor gets and which direction (clockwise/counterclockwise)
int PWMA2 = 11;  // Power supply for the "right motor"
int BIN1 = 12;   // The pin that moves the "right motor" of our build clockwise.
int BIN2 = 13;   // The pin that moves the "right motor" of our build counterclockwise.
// Global Variables (Lines 15-19 were Adapted from Sparkfun)
SparkFun_APDS9960 apds = SparkFun_APDS9960();  // Sets up the function the Sparkfun Light Color Sensor Needs
uint16_t ambient_light = 0;                    // Creates and ambient light variable to store ambient light data in (not used in this application, but it is available to you!)
// Creates a variable for the color sensor to store light subcategory data in (red, green, or blue)
uint16_t red_light = 0;
uint16_t green_light = 0;
uint16_t blue_light = 0;

// Define servo motor pin
const int servopin = 2;  // The servo motor is the motor responsible for setting a marker up or down, depending on whether a drawing function is activated.

void setup() {
  // Lines 28-39 were adapted from Sparkfun
  // Initiating serial port communication between the Arduino and your computer
  Serial.begin(9600);
  // Starting up color sensor
  if (apds.init()) {
    Serial.println(F("APDS-9960 initialization complete"));
  } else {
    Serial.println(F("Something went wrong during APDS-9960 init!"));
  }

  // Start running the color sensor
  if (apds.enableLightSensor(false)) {
    Serial.println(F("Light sensor is now running"));
  } else {
    Serial.println(F("Something went wrong during light sensor init!"));
  }

  // Wait for activation and calibration to finish
  delay(500);
  // Setting up the motor pins as outputs
  pinMode(AIN1, OUTPUT);
  pinMode(AIN2, OUTPUT);
  pinMode(PWMA1, OUTPUT);
  pinMode(BIN1, OUTPUT);
  pinMode(BIN2, OUTPUT);
  pinMode(PWMA2, OUTPUT);
  // Initialize servo motor pin as output
  myservo.attach(servopin);
}
void loop() {
  myservo.write(0);  // Sets the marker in the air (you may need to adjust the angle value for your servo to do this.)
  delay(1000);
  // Lines 61-74 were adapted from Sparkfun
  // Reading the light levels (ambient, red, green, blue)
  if (!apds.readAmbientLight(ambient_light) ||  // uses the color sensor to detect ambient light and store the numerical information into the ambient light variable (again, the ambient light data is not used for this project, put it is already programmed in if you want to use it.)
      // Using the color sensor to detect intensity of light in the red, green, and blue variables.
      !apds.readRedLight(red_light) || !apds.readGreenLight(green_light) || !apds.readBlueLight(blue_light)) {
    Serial.println("Error reading light values");  // Prints an error if the color sensor is not working correctly.
  } else {
    // Sending the color value data through serial communication (more so for your testing purposes rather than for the final product)
    Serial.print(" Red: ");
    Serial.print(red_light);
    Serial.print(" Green: ");
    Serial.print(green_light);
    Serial.print(" Blue: ");
    Serial.println(blue_light);
  }

  if (red_light >= 500) {           // We calibrated our color sensor's activation threshold based on the ambient light we saw in our testing grounds. You might want to consider running this code and checking the baseline color values in your environment to prevent automatic triggering of drawing functions.
    myservo.write(40);              // Sets the servo motor down to 40 degrees so that it lightly touches the ground.
    circles();                      // Activated the circle drawing function.
    Serial.println("Circles!");     // A confirmation via serial communication that the circle function was activated.
  } else if (green_light >= 500) {  //You may need to adjust your color sensor's green values to above the ambient levels to prevent accidental function triggers.
    myservo.write(40);              // Sets the servo motor attached to the marker down to the ground. You might want to adjust your degree value to better suit your servo's angle positions.
    triangle();                     // Activates the triangle drawing function
    Serial.println("Triangles!");   // A confirmation via serial communication that the triangle function was activated.
  } else if (blue_light >= 400) {
    myservo.write(40);           // Lowers the servo
    square();                    // Activates the sqaure drawing function
    Serial.println("Sqaures!");  // A confirmation via serial communication that the sqaure function was activated.
  }
}
void moveForward() {
  // Move both motors forward
  digitalWrite(AIN1, HIGH);
  /* If you want to flip the rotation of the motor's movement (make the motor move counterclockwise instead of clockwise),
  you would change the "HIGH" to "LOW" on the "IN1", and chnage the "LOW" to "HIGH" on "IN2" for motors A or B.
  */
  digitalWrite(AIN2, LOW);
  analogWrite(PWMA1, 255);  // If you want to lower the speed of both or one of your motors, you would chnage the "255" on this line and line 102 to a lower value.
  digitalWrite(BIN1, HIGH);
  digitalWrite(BIN2, LOW);
  analogWrite(PWMA2, 255);
}
void turnLeft() {
  // Spins the right motor forwards and stops the left motor
  digitalWrite(AIN1, HIGH);
  digitalWrite(AIN2, LOW);
  analogWrite(PWMA1, 255);
  digitalWrite(BIN1, LOW);
  digitalWrite(BIN2, LOW);
  analogWrite(PWMA2, 0);
}
void stopMotors() {
  // Stops all motors
  digitalWrite(AIN1, LOW);
  digitalWrite(AIN2, LOW);
  analogWrite(PWMA1, 0);
  digitalWrite(BIN1, LOW);
  digitalWrite(BIN2, LOW);
  analogWrite(PWMA2, 0);
}
void square() {
  // Four straight movements and four turns to the left
  moveForward();
  delay(1000);  // The 1 second forward movement of the motor determines length of each side of the square.
  turnLeft();
  delay(1350);  // The time the system turns left determines the angle of the turn. You may want to adjust this value to better fit your conditions.
  moveForward();
  delay(1000);
  turnLeft();
  delay(1350);
  moveForward();
  delay(1000);
  turnLeft();
  delay(1350);
  moveForward();
  delay(1000);
  turnLeft();
  delay(1350);
  stopMotors();
}

void triangle() {
  // A combination of three straight movements and three turns, one turn for each corner of a triangle.
  moveForward();
  delay(1000);  // The 1 second forward movement of the motor determines length of each side of the traingle.
  turnLeft();
  delay(1600);  // The time the system turns left determines the angle of the turn. We found this value to work well most of the time, but fine tuning the time could yield better results.
  moveForward();
  delay(1000);
  turnLeft();
  delay(1600);
  moveForward();
  delay(1000);
  turnLeft();
  delay(1600);
  stopMotors();
}
void circles() {
  // Right motor spins forward, left motor spins backward to rotate the robot in circles.
  digitalWrite(AIN1, HIGH);
  digitalWrite(AIN2, LOW);
  analogWrite(PWMA1, 255);
  digitalWrite(BIN1, LOW);
  digitalWrite(BIN2, HIGH);
  analogWrite(PWMA2, 255);
  delay(2125);  // "2125" ended up being the time we found the robot needed in this function to make one full circle (2125 milliseconds or 2.125 seconds)
  stopMotors();
}
//////////////////////////Extra functions (that we didn't use for this project) are also available (just remove the "/*" and "*/" to activate them)! //////////////////////////
/*
void moveBackward() {
  // Move both motors backward
  digitalWrite(AIN1, LOW);
  digitalWrite(AIN2, HIGH);
  analogWrite(PWMA1, 255);
  digitalWrite(BIN1, LOW);
  digitalWrite(BIN2, HIGH);
  analogWrite(PWMA2, 255);
*/
/*
void turnRight() {
  // Spins the left motor forwards and stops the right motor
  digitalWrite(AIN1, LOW);
  digitalWrite(AIN2, LOW);
  analogWrite(PWMA1, 0);
  digitalWrite(BIN1, LOW);
  digitalWrite(BIN2, HIGH);
  analogWrite(PWMA2, 255);
}
*/
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
